module lcd(clk,rst,din,cmd,start,rdy,dout,rs,rw,e);

input clk,rst,start;
input [2:0] cmd;
input [7:0] din;

output reg rdy;
output rs,rw,e;
output [7:0] dout;

reg start_i,register;
reg [7:0] data;
reg [3:0] state,nextstate;
reg [23:0] delay, delay_d;
reg [2:0]loop,loop_d;

wire done;

localparam init1=0,init2=1,init3=2,init4=3,init5=4,init6=5,init7=6,idle=7,clr1=8,clr2=9,stdelay=10,swrite=11,sline1=12,sline2=13;
parameter nop=0,clear=1,write=2,line1=3,line2=4;
parameter clkrate=50000000;
localparam clktime=1000000000/clkrate;

lcdw lcdw_i(
.clk (clk),
.rst (rst),
.din (data),
.r (register),
.start (start_i),
.dout (dout),
.rs (rs),
.rw (rw),
.e (e),
.done (done));
defparam lcdw_i.clkrate=clkrate;
always@(posedge clk)
begin
	if(!rst) begin
		state<=init1;
		delay<=24'd0;
		loop<=3'b0;
	end
	else begin
		state<=nextstate;
		delay<=delay_d;
		loop<=loop_d;
	end
end


always@*
begin
	rdy=1'b0;
	delay_d=0;
	data=0;
	start_i=0;
	loop_d=loop;
	register=1'b0;
	case(state)
		init1:begin
			if(delay==40000000/clktime)begin
				nextstate=init2;
				data=8'b00110000;
				start_i=1'b1;
			end else begin
				delay_d=delay+24'd1;
				nextstate=init1;
			end
		end	
		init2:begin //rcksetzen nach dem einschalten - 3x 00110000 senden und jeweils 40ms warten
			if(done)begin
				if(loop==3'd2)begin
					loop_d=3'd0;
					nextstate=init3;
				end else begin
					loop_d=loop+3'd1;
					nextstate=init1;
				end
			end else begin
				nextstate=init2;
			end
		end	
		init3:begin //init - 40us warten dann 8bit, 2 zeilen, zeichenraster 5*8
			if(delay==40000/clktime)begin
				nextstate=init4;
				data=8'b00111000;
				start_i=1'b1;
			end else begin
				delay_d=delay+24'd1;
				nextstate=init3;
			end
		end	
		init4:begin //init 40us warten dann display lschen
			if(delay==40000/clktime)begin
				nextstate=init5;
				data=8'b00000001;
				start_i=1'b1;
			end else begin
				delay_d=delay+24'd1;
				nextstate=init4;
			end
		end	
		init5:begin //init 2ms warten dann anzeige einschalten, cursor ein und blinken
			if(delay==2000000/clktime)begin
				nextstate=init6;
				data=8'b00001111;
				start_i=1'b1;
			end else begin
				delay_d=delay+24'd1;
				nextstate=init5;
			end
		end	
		init6:begin //init 40us warten dann adresszhler auf autoinkrement
			if(delay==40000/clktime)begin
				nextstate=init7;
				data=8'b00000110;
				start_i=1'b1;
			end else begin
				delay_d=delay+24'd1;
				nextstate=init6;
			end
		end	
		init7:begin //init nochmal 40us warten
			if(delay==40000/clktime)begin
				nextstate=idle;
			end else begin
				delay_d=delay+24'd1;
				nextstate=init7;
			end
		end	
		idle:begin //auf befehle warten
			if(start)begin
				case(cmd)
					write:begin
						nextstate=swrite;
					end
					clear:begin
						nextstate=clr1;
					end
					line1:begin
						nextstate=sline1;
					end
					line2:begin
						nextstate=sline2;
					end
					default:nextstate=idle;
				endcase
			end else begin			
			rdy=1'b1;
			nextstate=idle;
			end
		end
		swrite:begin //zeichen an cursorposition schreiben
			start_i=1'b1;
			register=1'b1;
			data=din;
			nextstate=stdelay;
		end
		sline1:begin //cursor an anfang zeile 1
			start_i=1'b1;
			data={1'b1,7'h00};
			nextstate=stdelay;
		end
		sline2:begin //cursor an anfang zeile 2
			start_i=1'b1;
			data={1'b1,7'h40};
			nextstate=stdelay;
		end
		stdelay:begin //40us wartezeit fr standard kommandos
			if(delay==40000/clktime)begin
				nextstate=idle;
			end else begin
				delay_d=delay+24'd1;
				nextstate=stdelay;
			end
		end	
		clr1:begin //display lschen
			start_i=1'b1;
			data=8'b00000001;
			nextstate=clr2;
		end
		clr2:begin //2ms wartezeit fr display-lsch-befehl
			if(delay==2000000/clktime)begin
				nextstate=idle;
			end else begin
				delay_d=delay+24'd1;
				nextstate=clr2;
			end
		end	
		
	endcase
end

endmodule

