//hier wird ein korrekter signalverlauf der steuersignale erzeugt.
//dabei werden die setup-, hold-, und zykluszeiten aus dem datenblatt eingehalten

module lcdw(clk,rst,din,r,start,dout,rs,rw,e,done);
input [7:0] din;
input start,clk,rst,r;
output reg [7:0] dout;
output reg rs,rw,e,done;

parameter clkrate=50000000;

localparam clktime=1000000000/clkrate;

//zeiten in ns siehe datenblatt
localparam tas=140;
localparam pweh=450;
localparam tah=10;
localparam tcyc=1000;

localparam idle=0,s1=1,s2=2,s3=3,s4=4;

reg [2:0] state,nextstate;
reg [6:0] delay, delay_d;
reg [7:0] dout_d;
reg rs_d;


always@(posedge clk)
begin
	if(!rst) begin
		state<=idle;
		delay<=7'd0;
		dout<=8'd0;
		rs<=1'b0;
	end
	else begin
		state<=nextstate;
		delay<=delay_d;
		dout<=dout_d;
		rs<=rs_d;
	end
end

always@*
begin
	done=1'b0;
	rw=1'b0;
	e=1'b0;
	rs_d=rs;
	dout_d=dout;
	delay_d=0;
	case(state)
		idle:begin
			done=1'b1;
			if(start==1'b1)begin
				nextstate=s1;
				dout_d=din;
				rs_d=r;				
			end else begin
			nextstate=idle;
			end
		end
		s1:begin
			rw=0;
			if(delay==tas/clktime)begin
				nextstate=s2;
			end else begin
				delay_d=delay+7'd1;
				nextstate=s1;
			end
		end
		s2:begin
			rw=0;
			e=1;
			if(delay==pweh/clktime)begin
				nextstate=s3;
			end else begin
				delay_d=delay+7'd1;
				nextstate=s2;
			end
		end
		s3:begin
			rw=0;
			if(delay==tah/clktime)begin
				nextstate=s4;
			end else begin
				delay_d=delay+7'd1;
				nextstate=s3;
			end
		end
		s4:begin
			if(delay==(tcyc-tas-pweh-tah)/clktime)begin
				nextstate=idle;
				rs_d=1'b0;
				dout_d=8'd0;
			end else begin
				delay_d=delay+7'd1;
				nextstate=s4;
			end
		end
		default:nextstate=idle;
	endcase
end	
endmodule
